package net.knarcraft.ffmpegconverter.converter;

import net.knarcraft.ffmpegconverter.container.FFMpegCommand;
import net.knarcraft.ffmpegconverter.container.StreamProbeResult;
import net.knarcraft.ffmpegconverter.converter.module.ConverterModule;
import net.knarcraft.ffmpegconverter.converter.module.DebugModule;
import net.knarcraft.ffmpegconverter.converter.module.ModuleExecutor;
import net.knarcraft.ffmpegconverter.converter.module.hardwarecoding.HardwareDecodeModule;
import net.knarcraft.ffmpegconverter.converter.module.mapping.MapAllModule;
import net.knarcraft.ffmpegconverter.converter.module.output.CopyAudioModule;
import net.knarcraft.ffmpegconverter.converter.module.output.CopyVideoModule;
import net.knarcraft.ffmpegconverter.converter.module.output.MovTextModule;
import net.knarcraft.ffmpegconverter.converter.module.output.SetOutputFileModule;
import net.knarcraft.ffmpegconverter.streams.AudioStream;
import net.knarcraft.ffmpegconverter.utility.FFMpegHelper;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.ArrayList;
import java.util.List;

/**
 * A converter to embed adjacent subtitles into mp4 files
 */
public class SubtitleEmbed extends AbstractConverter {

    /**
     * Initializes variables used by the abstract converter
     *
     * @param ffprobePath <p>Path/command to ffprobe.</p>
     * @param ffmpegPath  <p>Path/command to ffmpeg.</p>
     */
    public SubtitleEmbed(String ffprobePath, String ffmpegPath) {
        super(null);
        this.ffprobePath = ffprobePath;
        this.ffmpegPath = ffmpegPath;
    }

    @Override
    @NotNull
    public List<String> getValidFormats() {
        return List.of("mp4");
    }

    @Override
    @Nullable
    public FFMpegCommand generateConversionCommand(@NotNull String executable, @NotNull StreamProbeResult probeResult,
                                                   @NotNull String outFile) {
        FFMpegCommand command = FFMpegHelper.getFFMpegGeneralFileCommand(executable, probeResult.parsedFiles());
        List<ConverterModule> modules = new ArrayList<>();

        if (this.debug) {
            modules.add(new DebugModule());
        }

        modules.add(new MapAllModule<>(probeResult.parsedStreams()));
        modules.add(new HardwareDecodeModule());
        List<AudioStream> audioStreams = probeResult.getAudioStreams();
        setOutputIndexes(audioStreams);
        modules.add(new CopyAudioModule(audioStreams));
        modules.add(new CopyVideoModule());
        modules.add(new MovTextModule());
        modules.add(new SetOutputFileModule(outFile));

        new ModuleExecutor(command, modules).execute();
        return command;
    }

}
