package net.knarcraft.ffmpegconverter.converter.module.mapping;

import net.knarcraft.ffmpegconverter.container.FFMpegCommand;
import net.knarcraft.ffmpegconverter.converter.module.ConverterModule;
import net.knarcraft.ffmpegconverter.streams.SubtitleStream;
import net.knarcraft.ffmpegconverter.utility.FFMpegHelper;
import org.jetbrains.annotations.NotNull;

import java.util.List;

/**
 * A module for selecting and mapping the nth subtitle stream available
 */
public class NthSubtitleStreamModule implements ConverterModule {
    private final int n;
    private final List<SubtitleStream> allStreams;

    /**
     * Instantiates a new n-th video stream module
     *
     * @param allStreams <p>All available streams</p>
     * @param n          <p>The index of the stream the user wants</p>
     */
    public NthSubtitleStreamModule(@NotNull List<SubtitleStream> allStreams, int n) {
        this.allStreams = allStreams;
        this.n = n;
    }

    @Override
    public void addArguments(@NotNull FFMpegCommand command) {
        SubtitleStream subtitleStream = FFMpegHelper.getNthSteam(this.allStreams, this.n);
        if (subtitleStream != null) {
            FFMpegHelper.mapStream(command, subtitleStream);
        } else {
            throw new IllegalArgumentException("Selected subtitle stream does not exist.");
        }
    }

}
